#!/usr/bin/python
#
# Copyright (c) 2026 Microsoft
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_cognitiveservicesaccount_info
version_added: "3.21.0"
short_description: Get Azure Cognitive Services account information
description:
    - Retrieve information about Azure Cognitive Services accounts.
    - Can query a specific account, all accounts in a resource group, or all accounts in a subscription.
options:
    resource_group:
        description:
            - Name of the resource group.
            - Required when querying a specific account by name.
        type: str
    name:
        description:
            - Name of the Cognitive Services account.
            - When specified, returns information for this specific account.
        type: str
    show_keys:
        description:
            - Include API keys in the response.
            - Keys are sensitive - use C(no_log) on the task when retrieving keys.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get specific Cognitive Services account info
  azure.azcollection.azure_rm_cognitiveservicesaccount_info:
    resource_group: myResourceGroup
    name: mycontentsafety
  register: account_info

- name: Get account with API keys
  azure.azcollection.azure_rm_cognitiveservicesaccount_info:
    resource_group: myResourceGroup
    name: mycontentsafety
    show_keys: true
  register: account_with_keys
  no_log: true

- name: Use the endpoint
  debug:
    msg: "Endpoint: {{ account_info.accounts[0].endpoint }}"

- name: List all Cognitive Services accounts in resource group
  azure.azcollection.azure_rm_cognitiveservicesaccount_info:
    resource_group: myResourceGroup
  register: rg_accounts

- name: List all Cognitive Services accounts in subscription
  azure.azcollection.azure_rm_cognitiveservicesaccount_info:
  register: all_accounts
'''

RETURN = '''
accounts:
    description:
        - List of Cognitive Services accounts.
    returned: always
    type: list
    elements: dict
    sample: [
      {
          "etag": "\\"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx\\"",
          "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-csa/providers/Microsoft.CognitiveServices/accounts/csexxxxxxxxxxxxxxx",
          "identity": {
              "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "type": "SystemAssigned"
          },
          "kind": "ContentSafety",
          "location": "eastus",
          "name": "csexxxxxxxxxxxxxxx",
          "properties": {
              "allow_project_management": false,
              "call_rate_limit": {
                  "rules": [
                      {
                          "count": 10.0,
                          "dynamic_throttling_enabled": true,
                          "key": "ContentSafety.GPTFeature",
                          "match_patterns": [
                              {
                                  "method": "*",
                                  "path": "contentsafety/text:detectungroundedness*"
                              }
                          ],
                          "renewal_period": 10.0
                      },
                      {
                          "count": 5.0,
                          "key": "ContentSafety.All",
                          "match_patterns": [
                              {
                                  "method": "*",
                                  "path": "contentsafety/*"
                              }
                          ],
                          "renewal_period": 1.0
                      },
                      {
                          "count": 5.0,
                          "key": "default",
                          "match_patterns": [
                              {
                                  "method": "*",
                                  "path": "*"
                              }
                          ],
                          "renewal_period": 1.0
                      }
                  ]
              },
              "capabilities": [
                  {
                      "name": "VirtualNetworks"
                  },
                  {
                      "name": "EnabledApimVnet",
                      "value": "true"
                  },
                  {
                      "name": "Container",
                      "value": "ContentSafety.TextAnalyze,ContentSafety.ImageAnalyze,ContentSafety.Jailbreak,ContentSafety.jailbreakanalyze,ContentSafety.jailbreakanalyzenew"
                  }
              ],
              "date_created": "2026-06-24T18:41:21.7869275Z",
              "disable_local_auth": true,
              "endpoint": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/",
              "endpoints": {
                  "Container": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/",
                  "Content Safety": "https://csexxxxxxxxxxxxxxx-2b94a.cognitiveservices.azure.com/"
              },
              "internal_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
              "is_migrated": false,
              "private_endpoint_connections": [],
              "provisioning_state": "Succeeded",
              "public_network_access": "Enabled",
              "quota_limit": {
                  "rules": [
                      {
                          "count": 5000.0,
                          "key": "ContentSafety.All",
                          "match_patterns": [
                              {
                                  "method": "*",
                                  "path": "contentsafety/*"
                              }
                          ],
                          "renewal_period": 2592000.0
                      }
                  ]
              }
          },
          "sku": {
              "name": "F0"
          },
          "system_data": {
              "created_at": "2026-06-24T18:41:21.200238Z",
              "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "created_by_type": "Application",
              "last_modified_at": "2026-06-24T18:41:21.200238Z",
              "last_modified_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "last_modified_by_type": "Application"
          },
          "type": "Microsoft.CognitiveServices/accounts"
      }
    ]
'''  # NOQA

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMCognitiveServicesAccountInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str'),
            name=dict(type='str'),
            show_keys=dict(type='bool', default=False)
        )

        self.resource_group = None
        self.name = None
        self.show_keys = False

        self.results = dict(
            changed=False,
            accounts=[]
        )

        super(AzureRMCognitiveServicesAccountInfo, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False
        )

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        # Validate parameters
        if self.name and not self.resource_group:
            self.module.fail_json(msg="Parameter error: resource_group required when filtering by name")

        # Query accounts based on parameters
        if self.name:
            # Specific account
            results = self.get_account()
        elif self.resource_group:
            # All accounts in resource group
            results = self.list_by_resource_group()
        else:
            # All accounts in subscription
            results = self.list_all()

        self.results['accounts'] = results
        return self.results

    def get_account(self):
        """Get specific Cognitive Services account"""
        self.log('Getting Cognitive Services account {0}'.format(self.name))
        try:
            account_obj = self.cognitive_services_management_client.accounts.get(
                self.resource_group,
                self.name
            )
            account_dict = account_obj.as_dict()
            if self.show_keys:
                account_dict['auth_keys'] = self.get_keys(self.resource_group,
                                                          self.name)
            return [account_dict]
        except ResourceNotFoundError:
            self.log('Account {0} not found'.format(self.name))
            return []

    def list_by_resource_group(self):
        """List all Cognitive Services accounts in resource group"""
        self.log('Listing Cognitive Services accounts in resource group {0}'.format(self.resource_group))
        results = []
        try:
            accounts = self.cognitive_services_management_client.accounts.list_by_resource_group(
                self.resource_group
            )
            for account_obj in accounts:
                account_dict = account_obj.as_dict()
                if self.show_keys:
                    account_dict['auth_keys'] = self.get_keys(self.resource_group,
                                                              account_obj.name)
                results.append(account_dict)
        except Exception as exc:
            self.module.fail_json(msg='Error listing accounts: {0}'.format(str(exc)))
        return results

    def list_all(self):
        """List all Cognitive Services accounts in subscription"""
        self.log('Listing all Cognitive Services accounts in subscription')
        results = []
        try:
            accounts = self.cognitive_services_management_client.accounts.list()
            for account_obj in accounts:
                account_dict = account_obj.as_dict()
                if self.show_keys:
                    # Extract resource group from account ID
                    # Format: /subscriptions/{sub}/resourceGroups/{rg}/providers/...
                    id_parts = account_obj.id.split('/')
                    rg_index = id_parts.index('resourceGroups') + 1
                    account_dict['auth_keys'] = self.get_keys(id_parts[rg_index],
                                                              account_obj.name)
                results.append(account_dict)
        except Exception as exc:
            self.module.fail_json(msg='Error listing accounts: {0}'.format(str(exc)))
        return results

    def get_keys(self, resource_group, account_name):
        """Get API keys for the account"""
        try:
            keys_obj = self.cognitive_services_management_client.accounts.list_keys(
                resource_group,
                account_name
            )
            return {
                'key1': keys_obj.key1,
                'key2': keys_obj.key2
            }
        except Exception as exc:
            self.module.fail_json(msg='Error retrieving keys: {0}'.format(str(exc)))


def main():
    AzureRMCognitiveServicesAccountInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_interfaces_l3
short_description: Resource module for networks _appliance _interfaces l3
description:
  - Manage operations create, update and delete of the resource networks _appliance
    _interfaces l3.
  - Create wired L3 interface.
  - Delete wired L3 interface.
  - Update wired L3 interface.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  interfaceId:
    description: InterfaceId path parameter. Interface ID.
    type: str
  ipv4:
    description: IPv4 configuration.
    suboptions:
      address:
        description: IPv4 address.
        type: str
      subnet:
        description: IPv4 subnet in CIDR notation.
        type: str
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  port:
    description: Port configuration.
    suboptions:
      interface:
        description: Structured interface identifier for the port being modified.
        suboptions:
          name:
            description: Read-only full interface name for the port.
            type: str
          number:
            description: Leaf port number for the port.
            type: int
          slot:
            description: Slot number for the port.
            type: int
          subslot:
            description: Subslot number for the port.
            type: int
        type: dict
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createNetworkApplianceInterfacesL3
    description: Complete reference of the createNetworkApplianceInterfacesL3 API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-interfaces-l3
  - name: Cisco Meraki documentation for appliance deleteNetworkApplianceInterfacesL3
    description: Complete reference of the deleteNetworkApplianceInterfacesL3 API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-interfaces-l3
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceInterfacesL3
    description: Complete reference of the updateNetworkApplianceInterfacesL3 API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-interfaces-l3
notes:
  - SDK Method used are
    appliance.Appliance.create_network_appliance_interfaces_l3,
    appliance.Appliance.delete_network_appliance_interfaces_l3,
    appliance.Appliance.update_network_appliance_interfaces_l3,
  - Paths used are
    post /networks/{networkId}/appliance/interfaces/l3,
    delete /networks/{networkId}/appliance/interfaces/l3/{interfaceId},
    put /networks/{networkId}/appliance/interfaces/l3/{interfaceId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_interfaces_l3:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    ipv4:
      address: 192.0.2.1
      subnet: 192.0.2.0/24
    networkId: string
    port:
      interface:
        name: GigabitEthernet0/0/1
        number: 1
        slot: 0
        subslot: 0
- name: Update by id
  cisco.meraki.networks_appliance_interfaces_l3:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    interfaceId: string
    ipv4:
      address: 192.0.2.1
      subnet: 192.0.2.0/24
    networkId: string
    port:
      interface:
        name: GigabitEthernet0/0/1
        number: 1
        slot: 0
        subslot: 0
- name: Delete by id
  cisco.meraki.networks_appliance_interfaces_l3:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    interfaceId: string
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "interfaceId": "string",
      "ipv4": {
        "address": "string",
        "subnet": "string"
      },
      "port": {
        "interface": {
          "name": {},
          "slot": {},
          "subslot": {},
          "number": {}
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: mysql_partition

short_description: Manage MySQL table partitions

description:
  - Add, drop, reorganize, truncate, or run maintenance operations on MySQL table partitions.
  - Supports RANGE, RANGE COLUMNS, LIST, LIST COLUMNS, HASH, and KEY partition types.
  - The module auto-detects the partition method of the target table and generates the appropriate SQL syntax.
  - The module is limited to MySQL and fails on MariaDB.

version_added: '5.2.0'

options:
  table:
    description:
      - Name of the partitioned table to manage.
    type: str
    required: true
  schema:
    description:
      - Database (schema) containing the table.
      - If omitted, the current default database from the connection is used.
    type: str
  action:
    description:
      - Partition operation to perform.
      - V(add) adds a new partition. For RANGE and LIST types, O(name) and O(value) are required.
        For HASH and KEY types, O(number) is required instead.
      - V(drop) removes partitions. Only valid for RANGE and LIST types.
      - V(reorganize) splits or merges partitions. Only valid for RANGE and LIST types.
        Requires O(name) and O(into).
      - V(truncate) removes all rows from specified partitions without dropping them.
      - V(check), V(repair), V(analyze), and V(optimize) run maintenance operations on specified partitions.
    type: str
    required: true
    choices:
      - add
      - drop
      - reorganize
      - truncate
      - check
      - repair
      - analyze
      - optimize
  name:
    description:
      - Partition name or names to operate on.
      - For O(action=add) on RANGE and LIST partition types, provide exactly one partition name.
      - For O(action=drop), O(action=truncate), O(action=check), O(action=repair),
        O(action=analyze), and O(action=optimize), provide one or more partition names.
      - For O(action=reorganize), specifies the source partitions to reorganize.
      - For maintenance and truncate actions, a single-element list containing V(ALL) targets every partition.
    type: list
    elements: str
  value:
    description:
      - Partition boundary expression, written as raw SQL.
      - For RANGE partitions, this is the expression used in C(VALUES LESS THAN), for example
        V(2024), V(MAXVALUE), or V('2024-07-01').
      - For LIST partitions, this is the expression used in C(VALUES IN), for example V(7,8,9).
      - Required when O(action=add) for RANGE and LIST partition types.
    type: str
  number:
    description:
      - Number of partitions to add.
      - Used only when O(action=add) for HASH and KEY partition types.
    type: int
  into:
    description:
      - List of target partition definitions for reorganize.
      - Each element is a dictionary with O(into[].name) and O(into[].value) keys.
      - Required when O(action=reorganize).
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name of the target partition.
        type: str
        required: true
      value:
        description:
          - Partition boundary expression for the target partition, written as raw SQL.
        type: str
        required: true

notes:
  - MariaDB is not supported. The module fails with an error on MariaDB servers.
  - C(DROP PARTITION) is only supported for RANGE and LIST partition types.
    HASH and KEY partitions cannot be dropped individually.
  - C(REORGANIZE PARTITION) with value definitions is only supported for RANGE and LIST types.
  - The O(value) and O(into[].value) parameters accept raw SQL expressions.
    They are validated against common SQL injection patterns but are not parameterized.
  - Each invocation runs a single C(ALTER TABLE ... PARTITION) statement.
  - For O(action=add) on HASH or KEY partitions, each call increases the partition count and is not idempotent.
  - Reducing HASH and KEY partition counts with C(COALESCE PARTITION) is not currently supported.

attributes:
  check_mode:
    support: full
  idempotent:
    support: partial
    details:
      - O(action=add) for RANGE and LIST types is idempotent when the partition already exists.
      - O(action=drop) is idempotent when the partition does not exist.
      - O(action=reorganize) is idempotent when the target partition layout already matches O(into).
      - Maintenance operations and truncate always report C(changed=true).
      - O(action=add) for HASH and KEY types always reports C(changed=true).

seealso:
  - module: ansible.mysql.mysql_db
  - module: ansible.mysql.mysql_query
  - name: MySQL ALTER TABLE Partition Operations
    description: Complete reference of MySQL partition management statements.
    link: https://dev.mysql.com/doc/refman/8.4/en/alter-table-partition-operations.html

author:
  - Ron Gershburg (@rgershbu)

extends_documentation_fragment:
  - ansible.mysql.mysql
'''

EXAMPLES = r'''
- name: Add a RANGE partition for the next year
  ansible.mysql.mysql_partition:
    table: events
    schema: mydb
    action: add
    name: p2025
    value: "2026"
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Add a LIST partition for new regions
  ansible.mysql.mysql_partition:
    table: sales
    schema: mydb
    action: add
    name: p_south
    value: "10, 11, 12"
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Add partitions to a HASH-partitioned table
  ansible.mysql.mysql_partition:
    table: sessions
    schema: mydb
    action: add
    number: 4
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Drop an old partition
  ansible.mysql.mysql_partition:
    table: events
    schema: mydb
    action: drop
    name:
      - p2020
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Reorganize a partition into two
  ansible.mysql.mysql_partition:
    table: events
    schema: mydb
    action: reorganize
    name:
      - p2024
    into:
      - name: p2024h1
        value: "2024"
      - name: p2024h2
        value: "2025"
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Truncate a partition
  ansible.mysql.mysql_partition:
    table: events
    schema: mydb
    action: truncate
    name:
      - p2020
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Analyze all partitions
  ansible.mysql.mysql_partition:
    table: events
    schema: mydb
    action: analyze
    name:
      - ALL
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Monthly partition rotation
  block:
    - name: Add next month partition
      ansible.mysql.mysql_partition:
        table: events
        schema: mydb
        action: add
        name: p202502
        value: "'2025-03-01'"
        login_unix_socket: /run/mysqld/mysqld.sock

    - name: Drop oldest partition
      ansible.mysql.mysql_partition:
        table: events
        schema: mydb
        action: drop
        name:
          - p202401
        login_unix_socket: /run/mysqld/mysqld.sock
'''

RETURN = r'''
queries:
  description: List of SQL queries executed (or that would be executed in check mode).
  returned: always
  type: list
  elements: str
  sample: ["ALTER TABLE `mydb`.`events` ADD PARTITION (PARTITION `p2025` VALUES LESS THAN (2026))"]
partition_info:
  description: Partition metadata from C(INFORMATION_SCHEMA.PARTITIONS) after the operation.
  returned: success
  type: list
  elements: dict
  contains:
    PARTITION_NAME:
      description: Name of the partition.
      returned: success
      type: str
      sample: p2025
    PARTITION_METHOD:
      description: Partitioning method used by the table.
      returned: success
      type: str
      sample: RANGE
    PARTITION_EXPRESSION:
      description: Expression used by the table partition definition.
      returned: success
      type: str
      sample: created_year
    PARTITION_DESCRIPTION:
      description: Partition boundary or value list as reported by C(INFORMATION_SCHEMA.PARTITIONS).
      returned: success
      type: str
      sample: '2026'
    PARTITION_ORDINAL_POSITION:
      description: Position of the partition within the table definition.
      returned: success
      type: int
      sample: 4
    TABLE_ROWS:
      description: Estimated number of rows stored in the partition.
      returned: success
      type: int
      sample: 0
msg:
  description: Human-readable description of the result.
  returned: always
  type: str
  sample: "Partition p2025 added."
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native
from ansible_collections.ansible.mysql.plugins.module_utils.database import (
    check_input,
    mysql_quote_identifier,
)
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    get_server_implementation,
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)


PARTITION_QUERY = (
    "SELECT PARTITION_NAME, PARTITION_METHOD, PARTITION_EXPRESSION, "
    "PARTITION_DESCRIPTION, PARTITION_ORDINAL_POSITION, TABLE_ROWS "
    "FROM INFORMATION_SCHEMA.PARTITIONS "
    "WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s "
    "AND PARTITION_NAME IS NOT NULL "
    "AND SUBPARTITION_NAME IS NULL "
    "ORDER BY PARTITION_ORDINAL_POSITION"
)

HASH_KEY_METHODS = frozenset(('HASH', 'LINEAR HASH', 'KEY', 'LINEAR KEY'))
RANGE_METHODS = frozenset(('RANGE', 'RANGE COLUMNS'))
LIST_METHODS = frozenset(('LIST', 'LIST COLUMNS'))
MAINTENANCE_ACTIONS = frozenset(('check', 'repair', 'analyze', 'optimize'))


def quote_partition(name):
    return '`%s`' % name.replace('`', '``')


def get_table_ref(schema, table):
    if schema:
        return mysql_quote_identifier('%s.%s' % (schema, table), 'table')
    return mysql_quote_identifier(table, 'table')


def get_current_schema(cursor):
    cursor.execute("SELECT DATABASE()")
    row = cursor.fetchone()
    if row:
        return row.get('DATABASE()')
    return None


def get_partition_info(cursor, schema, table):
    cursor.execute(PARTITION_QUERY, (schema, table))
    return [dict(row) for row in cursor.fetchall()]


def partition_exists(partitions, name):
    return any(p['PARTITION_NAME'] == name for p in partitions)


def get_partition(partitions, name):
    for partition in partitions:
        if partition['PARTITION_NAME'] == name:
            return partition
    return None


def get_partition_method(partitions):
    if not partitions:
        return None
    return partitions[0]['PARTITION_METHOD']


def normalize_partition_description(value):
    return ','.join(str(value).strip().split(','))


def reorganize_matches_current_state(current_partitions, source_names, into):
    if any(partition_exists(current_partitions, source_name) for source_name in source_names):
        return False

    for target in into:
        current_partition = get_partition(current_partitions, target['name'])
        if current_partition is None:
            return False
        if normalize_partition_description(current_partition['PARTITION_DESCRIPTION']) != normalize_partition_description(target['value']):
            return False

    return True


def build_add_query(table_ref, partition_method, partition_name, value, number):
    if partition_method in HASH_KEY_METHODS:
        return 'ALTER TABLE %s ADD PARTITION PARTITIONS %d' % (table_ref, number)

    pname = quote_partition(partition_name)
    if partition_method in RANGE_METHODS:
        return 'ALTER TABLE %s ADD PARTITION (PARTITION %s VALUES LESS THAN (%s))' % (
            table_ref, pname, value)

    return 'ALTER TABLE %s ADD PARTITION (PARTITION %s VALUES IN (%s))' % (
        table_ref, pname, value)


def build_drop_query(table_ref, partitions):
    quoted = ', '.join(quote_partition(p) for p in partitions)
    return 'ALTER TABLE %s DROP PARTITION %s' % (table_ref, quoted)


def build_reorganize_query(table_ref, partition_method, source_partitions, into):
    source = ', '.join(quote_partition(p) for p in source_partitions)

    defs = []
    for part in into:
        pname = quote_partition(part['name'])
        val = part['value']
        if partition_method in RANGE_METHODS:
            defs.append('PARTITION %s VALUES LESS THAN (%s)' % (pname, val))
        else:
            defs.append('PARTITION %s VALUES IN (%s)' % (pname, val))

    return 'ALTER TABLE %s REORGANIZE PARTITION %s INTO (%s)' % (
        table_ref, source, ', '.join(defs))


def build_truncate_query(table_ref, partitions):
    if len(partitions) == 1 and partitions[0].upper() == 'ALL':
        return 'ALTER TABLE %s TRUNCATE PARTITION ALL' % table_ref
    quoted = ', '.join(quote_partition(p) for p in partitions)
    return 'ALTER TABLE %s TRUNCATE PARTITION %s' % (table_ref, quoted)


def build_maintenance_query(table_ref, action, partitions):
    if len(partitions) == 1 and partitions[0].upper() == 'ALL':
        return 'ALTER TABLE %s %s PARTITION ALL' % (table_ref, action.upper())
    quoted = ', '.join(quote_partition(p) for p in partitions)
    return 'ALTER TABLE %s %s PARTITION %s' % (table_ref, action.upper(), quoted)


def validate_inputs(module, action, partition_method, names, value, number, into):
    if action == 'add':
        if partition_method in HASH_KEY_METHODS:
            if number is None:
                module.fail_json(msg='number is required when adding %s partitions.' % partition_method)
            if number < 1:
                module.fail_json(msg='number must be at least 1.')
        else:
            if not names:
                module.fail_json(msg='name is required when adding %s partitions.' % partition_method)
            if len(names) != 1:
                module.fail_json(msg='exactly one name is required when adding %s partitions.' % partition_method)
            if value is None:
                module.fail_json(msg='value is required when adding %s partitions.' % partition_method)

    elif action == 'drop':
        if not names:
            module.fail_json(msg='name is required for drop.')
        if partition_method in HASH_KEY_METHODS:
            module.fail_json(
                msg='DROP PARTITION is not supported for %s partitions.' % partition_method)

    elif action == 'reorganize':
        if not names:
            module.fail_json(msg='name is required for reorganize (source partitions).')
        if not into:
            module.fail_json(msg='into is required for reorganize (target partition definitions).')
        if partition_method in HASH_KEY_METHODS:
            module.fail_json(
                msg='REORGANIZE PARTITION with value definitions is not supported for %s partitions.' % partition_method)

    elif action == 'truncate' or action in MAINTENANCE_ACTIONS:
        if not names:
            module.fail_json(msg='name is required for %s.' % action)


def handle_add(module, cursor, table_ref, schema, table, partition_method, current_partitions):
    names = module.params['name'] or []
    value = module.params['value']
    number = module.params['number']

    if partition_method in HASH_KEY_METHODS:
        query = build_add_query(table_ref, partition_method, None, None, number)
    else:
        partition_name = names[0]
        if partition_exists(current_partitions, partition_name):
            module.exit_json(
                changed=False, queries=[],
                partition_info=current_partitions,
                msg='Partition %s already exists.' % partition_name)

        query = build_add_query(table_ref, partition_method, partition_name, value, None)

    queries = [query]
    if module.check_mode:
        module.exit_json(
            changed=True, queries=queries,
            partition_info=current_partitions,
            msg='Partition would be added.')

    try:
        cursor.execute(query)
    except Exception as e:
        module.fail_json(msg='Failed to add partition: %s' % to_native(e), queries=queries)

    return queries


def handle_drop(module, cursor, table_ref, schema, table, current_partitions):
    partitions = module.params['name']

    existing = [p for p in partitions if partition_exists(current_partitions, p)]
    if not existing:
        module.exit_json(
            changed=False, queries=[],
            partition_info=current_partitions,
            msg='No matching partitions to drop.')

    query = build_drop_query(table_ref, existing)
    queries = [query]

    if module.check_mode:
        module.exit_json(
            changed=True, queries=queries,
            partition_info=current_partitions,
            msg='Partitions would be dropped.')

    try:
        cursor.execute(query)
    except Exception as e:
        module.fail_json(msg='Failed to drop partition: %s' % to_native(e), queries=queries)

    return queries


def handle_reorganize(module, cursor, table_ref, partition_method, current_partitions):
    partitions = module.params['name']
    into = module.params['into']

    if reorganize_matches_current_state(current_partitions, partitions, into):
        module.exit_json(
            changed=False, queries=[],
            partition_info=current_partitions,
            msg='Partitions already match requested layout.')

    missing = [partition for partition in partitions if not partition_exists(current_partitions, partition)]
    if missing:
        module.fail_json(
            msg='Source partitions do not exist: %s.' % ', '.join(missing),
            partition_info=current_partitions)

    query = build_reorganize_query(table_ref, partition_method, partitions, into)
    queries = [query]

    if module.check_mode:
        module.exit_json(
            changed=True, queries=queries,
            partition_info=current_partitions,
            msg='Partitions would be reorganized.')

    try:
        cursor.execute(query)
    except Exception as e:
        module.fail_json(msg='Failed to reorganize partitions: %s' % to_native(e), queries=queries)

    return queries


def handle_truncate(module, cursor, table_ref, current_partitions):
    partitions = module.params['name']

    query = build_truncate_query(table_ref, partitions)
    queries = [query]

    if module.check_mode:
        module.exit_json(
            changed=True, queries=queries,
            partition_info=current_partitions,
            msg='Partitions would be truncated.')

    try:
        cursor.execute(query)
    except Exception as e:
        module.fail_json(msg='Failed to truncate partition: %s' % to_native(e), queries=queries)

    return queries


def handle_maintenance(module, cursor, table_ref, action, current_partitions):
    partitions = module.params['name']

    query = build_maintenance_query(table_ref, action, partitions)
    queries = [query]

    if module.check_mode:
        module.exit_json(
            changed=True, queries=queries,
            partition_info=current_partitions,
            msg='%s PARTITION would be executed.' % action.upper())

    try:
        cursor.execute(query)
    except Exception as e:
        module.fail_json(msg='Failed to %s partition: %s' % (action, to_native(e)), queries=queries)

    return queries


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        table=dict(type='str', required=True),
        schema=dict(type='str'),
        action=dict(type='str', required=True, choices=[
            'add', 'drop', 'reorganize', 'truncate',
            'check', 'repair', 'analyze', 'optimize',
        ]),
        name=dict(type='list', elements='str'),
        value=dict(type='str'),
        number=dict(type='int'),
        into=dict(type='list', elements='dict', options=dict(
            name=dict(type='str', required=True),
            value=dict(type='str', required=True),
        )),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    action = module.params['action']
    table = module.params['table']
    schema = module.params['schema']

    check_input(module, table, schema, module.params['value'])
    if module.params['name']:
        check_input(module, module.params['name'])
    if module.params['into']:
        for item in module.params['into']:
            check_input(module, item['name'], item['value'])

    try:
        cursor, db_conn = mysql_connect(
            module,
            module.params['login_user'],
            module.params['login_password'],
            module.params['config_file'],
            module.params['client_cert'],
            module.params['client_key'],
            module.params['ca_cert'],
            connect_timeout=module.params['connect_timeout'],
            check_hostname=module.params['check_hostname'],
            cursor_class='DictCursor',
            autocommit=True,
        )
    except Exception as e:
        module.fail_json(msg='unable to connect to database: %s' % to_native(e))

    if get_server_implementation(module, cursor) != 'mysql':
        module.fail_json(msg='mysql_partition is supported only by MySQL. MariaDB is not supported.')

    if not schema:
        schema = get_current_schema(cursor)
        if not schema:
            module.fail_json(msg='No database selected and schema parameter not specified.')

    current_partitions = get_partition_info(cursor, schema, table)
    if not current_partitions:
        module.fail_json(msg='Table %s.%s does not exist or is not partitioned.' % (schema, table))

    partition_method = get_partition_method(current_partitions)
    table_ref = get_table_ref(schema, table)

    validate_inputs(
        module, action, partition_method,
        module.params['name'], module.params['value'],
        module.params['number'], module.params['into'])

    if action == 'add':
        queries = handle_add(module, cursor, table_ref, schema, table, partition_method, current_partitions)
    elif action == 'drop':
        queries = handle_drop(module, cursor, table_ref, schema, table, current_partitions)
    elif action == 'reorganize':
        queries = handle_reorganize(module, cursor, table_ref, partition_method, current_partitions)
    elif action == 'truncate':
        queries = handle_truncate(module, cursor, table_ref, current_partitions)
    else:
        queries = handle_maintenance(module, cursor, table_ref, action, current_partitions)

    updated_partitions = get_partition_info(cursor, schema, table)

    module.exit_json(
        changed=True,
        queries=queries,
        partition_info=updated_partitions,
        msg='%s partition operation completed.' % action.upper(),
    )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible community
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_tablespace

short_description: Manage MySQL InnoDB general tablespaces

description:
  - Create, update, rename, or drop MySQL InnoDB general tablespaces.
  - Supports general tablespace lifecycle operations on MySQL servers.
  - Key rotation is outside this module's scope because MySQL exposes it through
    C(ALTER INSTANCE ROTATE INNODB MASTER KEY), not as a per-tablespace lifecycle action.

version_added: '5.2.0'

options:
  name:
    description:
      - Name of the tablespace to manage.
    type: str
    required: true
  state:
    description:
      - If V(present), create the tablespace when it does not exist.
      - If V(present), alter mutable tablespace attributes when it already exists.
      - If V(absent), drop the tablespace.
    type: str
    choices: [absent, present]
    default: present
  datafile:
    description:
      - Tablespace datafile path.
      - Create-only.
      - Required on MySQL versions earlier than V(8.0.14).
      - Optional on MySQL V(8.0.14) and later.
    type: str
  file_block_size:
    description:
      - File block size for the general tablespace.
      - Create-only.
      - Existing tablespaces are compared best-effort using available metadata.
      - Deeper semantic validation is left to MySQL because valid values depend on
        C(innodb_page_size) and compressed-page rules rather than a standalone whitelist.
    type: int
  encryption:
    description:
      - Whether the general tablespace should be encrypted.
      - Uses MySQL literal values V(Y) or V(N).
      - Supported on MySQL V(8.0.13) and later.
      - Can be set during create or alter.
      - Does not rotate InnoDB master keys. Key rotation remains an instance-level
        operation exposed through C(ALTER INSTANCE ROTATE INNODB MASTER KEY).
    type: str
  rename_to:
    description:
      - Rename the tablespace to this new name.
      - Alter-only.
      - Supported on MySQL V(8.0.3) and later.
      - Cannot be used when O(state) is V(absent).
    type: str
  autoextend_size:
    description:
      - Tablespace autoextend size in bytes.
      - Supported on MySQL V(8.0.23) and later.
      - Can be set during create or alter.
    type: int

notes:
  - Compatible with MySQL only.
  - O(datafile) and O(file_block_size) are create-only options.
  - O(rename_to) is an alter-only option.
  - Key rotation is outside this module's scope because MySQL exposes it with
    C(ALTER INSTANCE ROTATE INNODB MASTER KEY).
  - Metadata reads used for idempotency and post-change verification may require the C(PROCESS) privilege.
  - C(DROP TABLESPACE) requires the tablespace to be empty.

attributes:
  check_mode:
    support: full
    details:
      - In check mode the module reads current tablespace state and reports the DDL it would run without executing it.
  idempotent:
    support: full
    details:
      - The module emits DDL only when the requested tablespace lifecycle state differs from the current server state.

seealso:
  - module: ansible.mysql.mysql_tablespace_info
  - name: MySQL ALTER INSTANCE reference
    description: Reference for instance-level operations such as C(ALTER INSTANCE ROTATE INNODB MASTER KEY).
    link: https://dev.mysql.com/doc/refman/8.4/en/alter-instance.html
  - name: MySQL CREATE TABLESPACE reference
    description: Complete reference of the CREATE TABLESPACE command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/create-tablespace.html
  - name: MySQL ALTER TABLESPACE reference
    description: Complete reference of the ALTER TABLESPACE command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/alter-tablespace.html
  - name: MySQL DROP TABLESPACE reference
    description: Complete reference of the DROP TABLESPACE command documentation.
    link: https://dev.mysql.com/doc/refman/8.4/en/drop-tablespace.html

author:
  - Ron Gershburg (@ronger4)

extends_documentation_fragment:
  - ansible.mysql.mysql
'''

EXAMPLES = r'''
# If you encounter the "Please explicitly state intended protocol" error,
# use the login_unix_socket argument
- name: Create a MySQL general tablespace
  ansible.mysql.mysql_tablespace:
    name: app_data
    datafile: ./app_data.ibd
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Create a MySQL general tablespace with file block and autoextend settings
  ansible.mysql.mysql_tablespace:
    name: analytics_data
    datafile: ./analytics_data.ibd
    file_block_size: 8192
    autoextend_size: 4194304

- name: Enable encryption on a MySQL tablespace
  ansible.mysql.mysql_tablespace:
    name: app_data
    encryption: Y

- name: Rename a MySQL tablespace
  ansible.mysql.mysql_tablespace:
    name: app_data
    rename_to: archive_data

- name: Drop an empty MySQL tablespace
  ansible.mysql.mysql_tablespace:
    name: archive_data
    state: absent
'''

RETURN = r'''
queries:
  description: List of executed queries.
  returned: when changed
  type: list
  sample:
    - CREATE TABLESPACE `app_data` ADD DATAFILE './app_data.ibd' ENGINE = InnoDB
tablespace:
  description:
    - Normalized representation of the MySQL tablespace after module execution when server metadata is available.
    - In create check mode, this is a predicted subset derived from requested values because no post-create server metadata exists yet.
  returned: when O(state) is V(present)
  type: dict
  contains:
    server_implementation:
      description: Server implementation that produced the result.
      type: str
      sample: mysql
    name:
      description: Tablespace name after module execution.
      type: str
      sample: app_data
    datafile:
      description: Tablespace datafile path when available.
      type: str
      sample: ./app_data.ibd
    autoextend_size:
      description: Autoextend size in bytes when available.
      type: int
      sample: 4194304
    encryption:
      description: Encryption state when available.
      type: str
      sample: N
    page_size:
      description: InnoDB page size in bytes when available.
      type: int
      sample: 16384
    zip_page_size:
      description: Compressed page size in bytes when available.
      type: int
      sample: 8192
    attached_tables:
      description: Tables attached to the tablespace when the server exposes them.
      type: list
      elements: str
      sample:
        - app/orders
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.ansible.mysql.plugins.module_utils.database import (
    check_input,
    mysql_quote_identifier,
)
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)
from ansible_collections.ansible.mysql.plugins.module_utils.tablespace import (
    ensure_tablespaces_supported,
    get_mysql_tablespace,
)

AUTOEXTEND_SIZE_INCREMENT = 4 * 1024 * 1024
MYSQL_TABLESPACE_RENAME_MIN_VERSION = (8, 0, 3)
MYSQL_TABLESPACE_ENCRYPTION_MIN_VERSION = (8, 0, 13)
MYSQL_TABLESPACE_OPTIONAL_DATAFILE_MIN_VERSION = (8, 0, 14)
MYSQL_TABLESPACE_AUTOEXTEND_SIZE_MIN_VERSION = (8, 0, 23)


def normalize_tablespace_encryption(encryption):
    if encryption is None:
        return None

    encryption = encryption.upper()
    if encryption not in ('Y', 'N'):
        raise ValueError("encryption must be either 'Y' or 'N'")

    return encryption


def validate_autoextend_size_input(autoextend_size):
    if autoextend_size is None:
        return None

    if autoextend_size < 0 or autoextend_size % AUTOEXTEND_SIZE_INCREMENT != 0:
        raise ValueError('autoextend_size must be a non-negative multiple of 4MB (4194304 bytes)')

    return autoextend_size


def validate_file_block_size_input(file_block_size):
    if file_block_size is None:
        return None

    if file_block_size <= 0:
        raise ValueError('file_block_size must be a positive integer')

    return file_block_size


def get_tablespace_file_block_size(tablespace):
    zip_page_size = tablespace.get('zip_page_size')
    # A ZIP page size of 0 means "not compressed", so the intentional falsy check
    # falls back to page_size instead of treating 0 as an actual file block size.
    if zip_page_size:
        return zip_page_size
    return tablespace.get('page_size')


def validate_tablespace_rename(server_version, rename):
    return _validate_version_gated_value(
        server_version,
        rename,
        MYSQL_TABLESPACE_RENAME_MIN_VERSION,
        'rename',
    )


def validate_tablespace_encryption(server_version, encryption):
    return _validate_version_gated_value(
        server_version,
        encryption,
        MYSQL_TABLESPACE_ENCRYPTION_MIN_VERSION,
        'encryption',
    )


def validate_tablespace_autoextend_size(server_version, autoextend_size):
    return _validate_version_gated_value(
        server_version,
        autoextend_size,
        MYSQL_TABLESPACE_AUTOEXTEND_SIZE_MIN_VERSION,
        'autoextend_size',
    )


def validate_tablespace_datafile(server_version, datafile):
    if datafile is None and server_version < MYSQL_TABLESPACE_OPTIONAL_DATAFILE_MIN_VERSION:
        raise ValueError(
            'datafile is required for MySQL versions earlier than %s'
            % _format_version(MYSQL_TABLESPACE_OPTIONAL_DATAFILE_MIN_VERSION)
        )
    return datafile


def _validate_version_gated_value(server_version, value, minimum_version, option_name):
    if value is None:
        return None
    if server_version < minimum_version:
        raise ValueError(
            '%s requires MySQL %s or later'
            % (option_name, _format_version(minimum_version))
        )
    return value


def _format_version(version):
    return '.'.join(str(part) for part in version)


def build_create_query(name, datafile=None, file_block_size=None, encryption=None, autoextend_size=None):
    query = ['CREATE TABLESPACE %s' % mysql_quote_identifier(name, 'role')]

    if datafile is not None:
        query.append('ADD DATAFILE %s' % quote_sql_value(datafile))

    if autoextend_size is not None:
        query.append('AUTOEXTEND_SIZE = %s' % autoextend_size)

    if file_block_size is not None:
        query.append('FILE_BLOCK_SIZE = %s' % file_block_size)

    if encryption is not None:
        query.append('ENCRYPTION = %s' % quote_sql_value(encryption))

    query.append('ENGINE = InnoDB')
    return ' '.join(query)


def build_alter_queries(current, rename_to=None, encryption=None, autoextend_size=None):
    queries = []
    target_name = current['name']

    if rename_to is not None and current['name'] != rename_to:
        queries.append(
            'ALTER TABLESPACE %s RENAME TO %s'
            % (
                mysql_quote_identifier(current['name'], 'role'),
                mysql_quote_identifier(rename_to, 'role'),
            )
        )
        target_name = rename_to

    if autoextend_size is not None and current.get('autoextend_size') != autoextend_size:
        queries.append(
            'ALTER TABLESPACE %s AUTOEXTEND_SIZE = %s'
            % (
                mysql_quote_identifier(target_name, 'role'),
                autoextend_size,
            )
        )

    if encryption is not None and current.get('encryption') != encryption:
        queries.append(
            'ALTER TABLESPACE %s ENCRYPTION = %s'
            % (
                mysql_quote_identifier(target_name, 'role'),
                quote_sql_value(encryption),
            )
        )

    return queries


def build_drop_query(name):
    return 'DROP TABLESPACE %s' % mysql_quote_identifier(name, 'role')


def quote_sql_value(value):
    if isinstance(value, bool):
        return '1' if value else '0'
    if isinstance(value, int):
        return str(value)
    return "'%s'" % str(value).replace("'", "''")


def execute_query(cursor, query):
    cursor.execute(query)


def predict_tablespace(current=None, name=None, datafile=None, encryption=None, autoextend_size=None, rename_to=None):
    if current is None:
        predicted = {
            'server_implementation': 'mysql',
            'name': name,
        }
    else:
        predicted = current.copy()

    if datafile is not None and 'datafile' not in predicted:
        predicted['datafile'] = datafile

    if encryption is not None:
        predicted['encryption'] = encryption

    if autoextend_size is not None:
        predicted['autoextend_size'] = autoextend_size

    if rename_to is not None:
        predicted['name'] = rename_to

    return predicted


def resolve_current_tablespace(module, cursor, server_version, name, rename_to=None):
    current = get_mysql_tablespace(cursor, server_version, name, module=module)

    if rename_to is None or rename_to == name:
        return current, False

    renamed = get_mysql_tablespace(cursor, server_version, rename_to, module=module)

    if current is None:
        if renamed is None:
            module.fail_json(
                msg=(
                    'rename_to is alter-only and requires an existing tablespace named %s, '
                    'or a tablespace already renamed to %s.'
                    % (name, rename_to)
                )
            )
        return renamed, True

    if renamed is not None:
        module.fail_json(
            msg='Cannot rename tablespace %s to %s because %s already exists.'
            % (name, rename_to, rename_to)
        )

    return current, False


def fail_if_rename_target_requires_changes(
    module,
    current,
    rename_to,
    datafile=None,
    file_block_size=None,
    encryption=None,
    autoextend_size=None,
):
    mismatches = []

    if datafile is not None and current.get('datafile') != datafile:
        mismatches.append('datafile')

    if file_block_size is not None:
        current_file_block_size = get_tablespace_file_block_size(current)
        if current_file_block_size is None or current_file_block_size != file_block_size:
            mismatches.append('file_block_size')

    if mismatches:
        module.fail_json(
            msg=(
                'rename_to target %s already exists but does not match the requested end state '
                '(%s). Refusing to treat the rename as already applied.'
                % (rename_to, ', '.join(mismatches))
            ),
            tablespace=current,
        )


def fail_if_create_only_options_differ(module, current, datafile=None, file_block_size=None):
    if datafile is not None and current.get('datafile') != datafile:
        module.fail_json(
            msg=(
                'datafile is create-only and cannot be changed for existing tablespace %s '
                '(current: %s, requested: %s).'
                % (current['name'], current.get('datafile'), datafile)
            ),
            tablespace=current,
        )

    if file_block_size is None:
        return

    current_file_block_size = get_tablespace_file_block_size(current)
    if current_file_block_size is None:
        module.fail_json(
            msg=(
                'Cannot compare create-only file_block_size for existing tablespace %s '
                'because current metadata does not expose page_size or zip_page_size.'
                % current['name']
            ),
            tablespace=current,
        )

    if current_file_block_size != file_block_size:
        module.fail_json(
            msg=(
                'file_block_size is create-only and cannot be changed for existing tablespace %s '
                '(current best-effort value: %s, requested: %s).'
                % (current['name'], current_file_block_size, file_block_size)
            ),
            tablespace=current,
        )


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        name=dict(type='str', required=True),
        state=dict(type='str', choices=['absent', 'present'], default='present'),
        datafile=dict(type='str'),
        file_block_size=dict(type='int'),
        encryption=dict(type='str'),
        rename_to=dict(type='str'),
        autoextend_size=dict(type='int'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    name = module.params['name']
    state = module.params['state']
    datafile = module.params['datafile']
    file_block_size = module.params['file_block_size']
    encryption = module.params['encryption']
    rename_to = module.params['rename_to']
    autoextend_size = module.params['autoextend_size']
    login_user = module.params['login_user']
    login_password = module.params['login_password']
    config_file = module.params['config_file']
    ssl_cert = module.params['client_cert']
    ssl_key = module.params['client_key']
    ssl_ca = module.params['ca_cert']
    connect_timeout = module.params['connect_timeout']
    check_hostname = module.params['check_hostname']

    if rename_to == name:
        rename_to = None

    if state == 'absent' and rename_to is not None:
        module.fail_json(msg='rename_to cannot be used with state=absent')

    check_input(module, name, datafile, rename_to, encryption)

    try:
        encryption = normalize_tablespace_encryption(encryption)
        autoextend_size = validate_autoextend_size_input(autoextend_size)
        file_block_size = validate_file_block_size_input(file_block_size)
    except ValueError as e:
        module.fail_json(msg=to_native(e))

    try:
        cursor, _db_conn = mysql_connect(
            module,
            login_user,
            login_password,
            config_file,
            ssl_cert,
            ssl_key,
            ssl_ca,
            connect_timeout=connect_timeout,
            check_hostname=check_hostname,
            cursor_class='DictCursor',
            autocommit=True,
        )
    except Exception as e:
        module.fail_json(msg='unable to connect to database: %s' % to_native(e))

    server_version = ensure_tablespaces_supported(module, cursor)
    queries = []

    try:
        rename_to = validate_tablespace_rename(server_version, rename_to)
        encryption = validate_tablespace_encryption(server_version, encryption)
        autoextend_size = validate_tablespace_autoextend_size(server_version, autoextend_size)
    except ValueError as e:
        module.fail_json(msg=to_native(e))

    if state == 'absent':
        current = get_mysql_tablespace(cursor, server_version, name, module=module)
        if current is None:
            module.exit_json(changed=False)

        query = build_drop_query(name)
        queries.append(query)

        if module.check_mode:
            module.exit_json(changed=True, queries=queries)

        try:
            execute_query(cursor, query)
        except Exception as e:
            module.fail_json(msg=to_native(e))

        module.exit_json(changed=True, queries=queries)

    current, rename_already_applied = resolve_current_tablespace(
        module,
        cursor,
        server_version,
        name,
        rename_to=rename_to,
    )

    if current is None:
        try:
            validate_tablespace_datafile(server_version, datafile)
        except ValueError as e:
            module.fail_json(msg=to_native(e))

        query = build_create_query(
            name,
            datafile=datafile,
            file_block_size=file_block_size,
            encryption=encryption,
            autoextend_size=autoextend_size,
        )
        queries.append(query)

        predicted = predict_tablespace(
            name=name,
            datafile=datafile,
            encryption=encryption,
            autoextend_size=autoextend_size,
        )

        if module.check_mode:
            module.exit_json(changed=True, queries=queries, tablespace=predicted)

        try:
            execute_query(cursor, query)
        except Exception as e:
            module.fail_json(msg=to_native(e))

        current = get_mysql_tablespace(cursor, server_version, name, module=module)
        module.exit_json(changed=True, queries=queries, tablespace=current or predicted)

    if rename_already_applied:
        fail_if_rename_target_requires_changes(
            module,
            current,
            rename_to,
            datafile=datafile,
            file_block_size=file_block_size,
            encryption=encryption,
            autoextend_size=autoextend_size,
        )
    else:
        fail_if_create_only_options_differ(
            module,
            current,
            datafile=datafile,
            file_block_size=file_block_size,
        )

    queries = build_alter_queries(
        current,
        rename_to=rename_to,
        encryption=encryption,
        autoextend_size=autoextend_size,
    )

    if not queries:
        module.exit_json(changed=False, tablespace=current)

    predicted = predict_tablespace(
        current=current,
        encryption=encryption,
        autoextend_size=autoextend_size,
        rename_to=rename_to,
    )

    if module.check_mode:
        module.exit_json(changed=True, queries=queries, tablespace=predicted)

    try:
        for query in queries:
            execute_query(cursor, query)
    except Exception as e:
        module.fail_json(msg=to_native(e))

    current = get_mysql_tablespace(cursor, server_version, predicted['name'], module=module)
    module.exit_json(changed=True, queries=queries, tablespace=current or predicted)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_fibre_channel_adapter_policy_info
short_description: Gather information about Fibre Channel Adapter Policies in Cisco Intersight
description:
  - Retrieve information about Fibre Channel Adapter Policies from L(Cisco Intersight,https://intersight.com).
  - Query policies by organization or policy name.
  - Returns structured data with policy metadata and Fibre Channel Adapter configuration details.
  - If no filters are provided, all Fibre Channel Adapter Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization to filter Fibre Channel Adapter Policies by.
      - Use 'default' for the default organization.
      - When specified, only policies from this organization will be returned.
    type: str
  name:
    description:
      - The exact name of the Fibre Channel Adapter Policy to retrieve information from.
      - When specified, only the matching policy will be returned.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
# Basic Usage Examples
- name: Fetch all Fibre Channel Adapter Policies from all organizations
  cisco.intersight.intersight_fibre_channel_adapter_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
  register: all_fc_adapter_policies

# Organization-specific Examples
- name: Fetch all Fibre Channel Adapter Policies from the default organization
  cisco.intersight.intersight_fibre_channel_adapter_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
  register: default_org_policies

- name: Fetch all Fibre Channel Adapter Policies from a custom organization
  cisco.intersight.intersight_fibre_channel_adapter_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
  register: engineering_policies

# Specific Policy Examples
- name: Fetch a specific Fibre Channel Adapter Policy by name
  cisco.intersight.intersight_fibre_channel_adapter_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "fc-adapter-policy-01"
  register: specific_policy

- name: Fetch a specific policy from a specific organization
  cisco.intersight.intersight_fibre_channel_adapter_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Production"
    name: "fc-adapter-policy-prod"
  register: specific_org_policy
'''

RETURN = r'''
api_response:
  description:
    - The API response containing Fibre Channel Adapter Policy information.
    - Returns a list.
  returned: always
  type: list
  sample:
    [
      {
        "Name": "fc-adapter-policy-01",
        "ObjectType": "vnic.FcAdapterPolicy",
        "Moid": "1234567890abcdef12345678",
        "Description": "Fibre Channel Adapter policy for production servers",
        "ErrorRecoverySettings": {
          "Enabled": true,
          "PortDownTimeout": 5000,
          "LinkDownTimeout": 20000,
          "IoRetryTimeout": 8,
          "IoRetryCount": 10
        },
        "ErrorDetectionTimeout": 5000,
        "ResourceAllocationTimeout": 15000,
        "FlogiSettings": {
          "Retries": 12,
          "Timeout": 8000
        },
        "PlogiSettings": {
          "Retries": 12,
          "Timeout": 30000
        },
        "InterruptSettings": {
          "Mode": "MSIx"
        },
        "IoThrottleCount": 1024,
        "LunCount": 4096,
        "LunQueueDepth": 254,
        "RxQueueSettings": {
          "RingSize": 128
        },
        "TxQueueSettings": {
          "RingSize": 128
        },
        "ScsiQueueSettings": {
          "Count": 8,
          "RingSize": 512
        },
        "Organization": {
          "Name": "default",
          "ObjectType": "organization.Organization",
          "Moid": "abcdef1234567890abcdef12"
        },
        "Tags": [
          {
            "Key": "Environment",
            "Value": "Production"
          },
          {
            "Key": "Owner",
            "Value": "Storage-Team"
          }
        ]
      }
    ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to fetch policy info
    resource_path = '/vnic/FcAdapterPolicies'

    # Get query parameters for policies
    query_params = intersight.set_query_params()

    # Get Fibre Channel Adapter policies
    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_umbrella_policies_remove
short_description: Resource module for networks _appliance _umbrella _policies _remove
description:
  - Manage operation create of the resource networks _appliance _umbrella _policies
    _remove. - > Remove one Cisco Umbrella DNS security policy from an MX network
    by policy ID. Returns 204 No Content on success. Behavior when the policy is not
    currently applied depends on the Cisco Umbrella API response.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  policy:
    description: Umbrella policy to remove.
    suboptions:
      id:
        description: Umbrella policy ID.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance removeNetworkApplianceUmbrellaPolicies
    description: Complete reference of the removeNetworkApplianceUmbrellaPolicies
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!remove-network-appliance-umbrella-policies
notes:
  - SDK Method used are
    appliance.Appliance.remove_network_appliance_umbrella_policies,
  - Paths used are
    post /networks/{networkId}/appliance/umbrella/policies/remove,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_umbrella_policies_remove:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
    policy:
      id: '13408726'
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible community
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mysql_tablespace_info

short_description: Gather MySQL tablespace information

description:
  - Gather general tablespace metadata from MySQL servers.
  - This module returns the general-tablespace view using
    C(INFORMATION_SCHEMA.FILES) together with the version-appropriate InnoDB metadata family.
  - On V(5.7), the module uses C(INFORMATION_SCHEMA.INNODB_SYS_TABLESPACES),
    C(INFORMATION_SCHEMA.INNODB_SYS_DATAFILES), and C(INFORMATION_SCHEMA.INNODB_SYS_TABLES).
  - On V(8.0) or later, the module uses C(INFORMATION_SCHEMA.INNODB_TABLESPACES),
    C(INFORMATION_SCHEMA.INNODB_DATAFILES), and C(INFORMATION_SCHEMA.INNODB_TABLES).
  - This module is read-only and always reports no change.

version_added: '5.2.0'

options:
  name:
    description:
      - Limit the collected information to a single tablespace name.
      - This matches the general tablespace name.
    type: str

notes:
  - Compatible with MySQL V(5.7.6) or later.
  - This module reports metadata only. Key rotation remains outside scope because
    MySQL exposes it through C(ALTER INSTANCE ROTATE INNODB MASTER KEY).
  - Tablespace metadata queries may require the C(PROCESS) privilege.
  - No external tools are required.

attributes:
  check_mode:
    support: full
  idempotent:
    support: full

seealso:
  - module: ansible.mysql.mysql_tablespace
  - name: MySQL INFORMATION_SCHEMA FILES reference
    description: Reference for C(INFORMATION_SCHEMA.FILES), used as the base MySQL metadata source.
    link: https://dev.mysql.com/doc/refman/8.4/en/information-schema-files-table.html

author:
  - Ron Gershburg (@ronger4)

extends_documentation_fragment:
  - ansible.mysql.mysql
'''

EXAMPLES = r'''
# If you encounter the "Please explicitly state intended protocol" error,
# use the login_unix_socket argument
- name: Gather MySQL tablespace information
  ansible.mysql.mysql_tablespace_info:
    login_unix_socket: /run/mysqld/mysqld.sock

- name: Gather one MySQL tablespace by name
  ansible.mysql.mysql_tablespace_info:
    login_unix_socket: /run/mysqld/mysqld.sock
    name: app_data
'''

RETURN = r'''
tablespaces:
  description:
    - List of normalized MySQL tablespace metadata rows.
    - Rows expose the general-tablespace view returned by the server.
  returned: always
  type: list
  elements: dict
  contains:
    name:
      description: Tablespace name.
      type: str
      sample: app_data
    space_id:
      description: Internal InnoDB space identifier.
      type: int
      sample: 17
    datafile:
      description: Tablespace datafile path reported by the server.
      returned: when available
      type: str
      sample: ./app_data.ibd
    extent_size:
      description: Extent size in bytes.
      returned: when available
      type: int
      sample: 1048576
    autoextend_size:
      description: Autoextend size in bytes.
      returned: when available
      type: int
      sample: 8388608
    maximum_size:
      description: Maximum size in bytes.
      returned: when available
      type: int
      sample: 67108864
    filesystem_block_size:
      description: Filesystem block size in bytes when available.
      returned: when available
      type: int
      sample: 4096
    page_size:
      description: InnoDB page size in bytes when available.
      returned: when available
      type: int
      sample: 16384
    file_size:
      description: Current file size in bytes when available.
      returned: when available
      type: int
      sample: 16777216
    allocated_size:
      description: Allocated size in bytes when available.
      returned: when available
      type: int
      sample: 8388608
    status:
      description: Tablespace status reported by the server.
      returned: when available
      type: str
      sample: NORMAL
    zip_page_size:
      description: Compressed page size in bytes.
      returned: when available
      type: int
      sample: 0
    state:
      description: Tablespace state.
      returned: when available
      type: str
      sample: active
    encryption:
      description: Encryption state.
      returned: when available
      type: str
      sample: N
    attached_tables:
      description: Tables attached to the tablespace when the server exposes them.
      type: list
      elements: str
      sample:
        - app/orders
        - app/order_items
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)
from ansible_collections.ansible.mysql.plugins.module_utils.tablespace import (
    ensure_tablespaces_supported,
    get_server_version_tuple,
    get_mysql_tablespaces,
)


def get_tablespaces_info(cursor, name=None, server_version=None, module=None):
    if server_version is None:
        server_version = get_server_version_tuple(cursor)

    return {
        'tablespaces': [
            format_tablespace_info(row)
            for row in get_mysql_tablespaces(
                cursor,
                server_version,
                name=name,
                module=module,
            )
        ]
    }


def format_tablespace_info(row):
    info = {
        'name': row['name'],
        'space_id': row['space_id'],
        'attached_tables': row['attached_tables'],
    }

    optional_fields = (
        'datafile',
        'extent_size',
        'autoextend_size',
        'maximum_size',
        'filesystem_block_size',
        'page_size',
        'file_size',
        'allocated_size',
        'zip_page_size',
        'status',
        'state',
        'encryption',
    )

    for field_name in optional_fields:
        if field_name in row and row[field_name] is not None:
            info[field_name] = row[field_name]

    return info


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        name=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    login_user = module.params['login_user']
    login_password = module.params['login_password']
    config_file = module.params['config_file']
    ssl_cert = module.params['client_cert']
    ssl_key = module.params['client_key']
    ssl_ca = module.params['ca_cert']
    connect_timeout = module.params['connect_timeout']
    check_hostname = module.params['check_hostname']
    name = module.params['name']

    try:
        cursor, _db_conn = mysql_connect(
            module,
            login_user,
            login_password,
            config_file,
            ssl_cert,
            ssl_key,
            ssl_ca,
            connect_timeout=connect_timeout,
            check_hostname=check_hostname,
            cursor_class='DictCursor',
        )
    except Exception as e:
        module.fail_json(msg='unable to connect to database: %s' % to_native(e))

    server_version = ensure_tablespaces_supported(module, cursor)

    module.exit_json(
        changed=False,
        **get_tablespaces_info(
            cursor,
            name=name,
            server_version=server_version,
            module=module,
        )
    )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = r'''
---
module: mariadb_password_policy

short_description: Manage MariaDB password policy settings

description:
  - This module only supports MariaDB; all MySQL-related options will be removed in the next releases. Use the C(ansible.mysql) collection for MySQL automation.
  - Manage password policy settings on MariaDB.
  - This module is configuration-only and does not install or uninstall password validation components or plugins.
  - On MySQL, the module manages C(validate_password) settings and related global password policy variables.
  - On MariaDB, the module manages the C(simple_password_check) plugin settings only.

author:
  - Steve Fulmer (@stevefulme1)
  - Ron Gershburg (@ronger4)

version_added: '5.2.0'

options:
  policy:
    description:
      - This option is scheduled for removal.
      - Password validation policy level on MySQL.
      - Supported only on MySQL.
    type: str
    choices: [low, medium, strong]
  length:
    description:
      - Minimum number of characters in a password.
      - Maps to the active password validation facility for the current engine.
    type: int
  mixed_case_count:
    description:
      - Minimum number of lowercase and uppercase letters required each by the active password validation facility.
      - Supported on MySQL and on MariaDB C(simple_password_check).
    type: int
  number_count:
    description:
      - Minimum number of numeric characters required.
      - Supported on MySQL and on MariaDB C(simple_password_check).
    type: int
  special_char_count:
    description:
      - Minimum number of non-alphanumeric characters required.
      - Supported on MySQL and on MariaDB C(simple_password_check).
    type: int
  check_user_name:
    description:
      - This option is scheduled for removal.
      - Whether passwords are checked against the user name.
      - Supported only on MySQL.
    type: bool
  password_lifetime:
    description:
      - This option is scheduled for removal.
      - Default password expiration lifetime in days.
      - Supported only on MySQL.
    type: int
  password_history:
    description:
      - This option is scheduled for removal.
      - Number of previous passwords that cannot be reused.
      - Supported only on MySQL.
    type: int
  reuse_interval:
    description:
      - This option is scheduled for removal.
      - Number of days before a password can be reused.
      - Supported only on MySQL.
    type: int
  mode:
    description:
      - This option is scheduled for removal.
      - How supported MySQL variables are set.
      - V(global) uses C(SET GLOBAL) and does not survive restarts by itself.
      - V(persist) uses C(SET PERSIST) on MySQL.
      - Supported only on MySQL.
    type: str
    choices: [global, persist]
    default: global

notes:
  - The required password validation component or plugin must already be enabled on the target server.
  - The module does not install C(validate_password) or MariaDB password validation plugins.
  - MariaDB support in the first iteration is limited to C(simple_password_check).

attributes:
  check_mode:
    support: full
  idempotent:
    support: full

seealso:
  - module: ansible.mariadb.mariadb_query
  - module: ansible.mariadb.mariadb_variables
  - name: MySQL password validation component
    description: Oracle MySQL reference for validate_password.
    link: https://dev.mysql.com/doc/refman/8.4/en/validate-password.html
  - name: MariaDB simple password check plugin
    description: MariaDB reference for simple_password_check.
    link: https://mariadb.com/docs/server/reference/plugins/password-validation-plugins/simple-password-check-plugin

extends_documentation_fragment:
  - ansible.mariadb.mysql
'''

EXAMPLES = r'''
- name: Configure shared password complexity settings on MySQL
  ansible.mariadb.mariadb_password_policy:
    login_unix_socket: /run/mysqld/mysqld.sock
    length: 12
    mixed_case_count: 2
    number_count: 2
    special_char_count: 1

- name: Configure MariaDB simple_password_check settings
  ansible.mariadb.mariadb_password_policy:
    login_unix_socket: /run/mysqld/mysqld.sock
    length: 14
    mixed_case_count: 2
    number_count: 2
    special_char_count: 2
'''

RETURN = r'''
queries:
  description: List of executed or predicted (in check mode) SQL statements.
  returned: always
  type: list
  elements: str
  sample:
    - SET GLOBAL `validate_password`.`length` = 12
settings:
  description: Normalized requested settings after execution or prediction (in check mode).
  returned: always
  type: dict
  sample:
    length: 12
    number_count: 2
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.ansible.mariadb.plugins.module_utils._version import LooseVersion
from ansible_collections.ansible.mariadb.plugins.module_utils.database import mysql_quote_identifier
from ansible_collections.ansible.mariadb.plugins.module_utils.mysql import (
    get_server_implementation,
    get_server_version,
    mysql_common_argument_spec,
    mysql_connect,
    mysql_driver,
    mysql_driver_fail_msg,
)


SETTING_VARIABLES = {
    'mysql': {
        'policy': 'validate_password.policy',
        'length': 'validate_password.length',
        'mixed_case_count': 'validate_password.mixed_case_count',
        'number_count': 'validate_password.number_count',
        'special_char_count': 'validate_password.special_char_count',
        'check_user_name': 'validate_password.check_user_name',
        'password_lifetime': 'default_password_lifetime',
        'password_history': 'password_history',
        'reuse_interval': 'password_reuse_interval',
    },
    'mariadb': {
        'length': 'simple_password_check_minimal_length',
        'mixed_case_count': 'simple_password_check_letters_same_case',
        'number_count': 'simple_password_check_digits',
        'special_char_count': 'simple_password_check_other_characters',
    },
}

MYSQL_ONLY_OPTIONS = (
    'policy',
    'check_user_name',
    'password_lifetime',
    'password_history',
    'reuse_interval',
)

INTEGER_OPTIONS = (
    'length',
    'mixed_case_count',
    'number_count',
    'special_char_count',
    'password_lifetime',
    'password_history',
    'reuse_interval',
)


def normalize_int_value(value):
    if isinstance(value, int):
        return value

    return int(value)


def get_variable(cursor, mysqlvar):
    cursor.execute("SHOW GLOBAL VARIABLES WHERE Variable_name = %s", (mysqlvar,))
    rows = cursor.fetchall()
    return rows[0][1] if rows else None


def set_variable(cursor, mysqlvar, value, mode):
    query = "SET %s %s = " % (mode.upper(), mysql_quote_identifier(mysqlvar, 'vars'))
    cursor.execute(query + "%s", (value,))


def normalize_bool_setting_value(value):
    if isinstance(value, str):
        value = value.upper()

    if value in ('ON', '1', 1, True):
        return 'ON'
    if value in ('OFF', '0', 0, False):
        return 'OFF'
    return value


def normalize_policy_value(value):
    value = str(value).strip().lower()
    numeric_policy = {
        '0': 'low',
        '1': 'medium',
        '2': 'strong',
    }
    return numeric_policy.get(value, value)


class MySQLPasswordPolicy(object):
    def __init__(self, module, cursor, server_implementation):
        self.module = module
        self.cursor = cursor
        self.server_implementation = server_implementation

    def configure(self, desired_settings, mode='global', check_mode=False):
        self.validate_supported_settings(desired_settings, mode)

        effective_settings = {}
        planned_changes = []

        for setting_name, desired_value in desired_settings.items():
            if desired_value is None:
                continue

            variable_name, current_value = self.read_setting(setting_name)
            normalized_current = self.normalize_value(setting_name, current_value)
            normalized_desired = self.normalize_value(setting_name, desired_value)

            effective_settings[setting_name] = normalized_desired

            if normalized_current != normalized_desired:
                planned_changes.append(
                    (setting_name, variable_name, self.query_value(setting_name, normalized_desired))
                )

        queries = []
        changed = bool(planned_changes)

        if check_mode:
            for _setting_name, variable_name, query_value in planned_changes:
                queries.append(self.format_set_query(variable_name, query_value, mode))
            return dict(changed=changed, queries=queries, settings=effective_settings)

        for _setting_name, variable_name, query_value in planned_changes:
            try:
                set_variable(self.cursor, variable_name, query_value, mode)
            except Exception as exc:
                self.module.fail_json(
                    msg='Failed to set %s: %s' % (variable_name, to_native(exc)),
                    changed=False,
                )
            queries.append(self.format_set_query(variable_name, query_value, mode))

        return dict(changed=changed, queries=queries, settings=effective_settings)

    def validate_supported_settings(self, desired_settings, mode):
        if self.server_implementation == 'mariadb':
            if mode == 'persist':
                self.module.fail_json(msg='mode=persist is supported only on MySQL.', changed=False)

            for option_name in MYSQL_ONLY_OPTIONS:
                if desired_settings.get(option_name) is not None:
                    self.module.fail_json(
                        msg='Parameter "%s" is supported only on MySQL.' % option_name,
                        changed=False,
                    )

    def read_setting(self, setting_name):
        variable_name = SETTING_VARIABLES[self.server_implementation][setting_name]
        value = get_variable(self.cursor, variable_name)
        if value is not None:
            return variable_name, value

        self.module.fail_json(
            msg='Password policy setting "%s" is not available on this server.' % setting_name,
            changed=False,
        )

    def normalize_value(self, setting_name, value):
        if setting_name == 'policy':
            return normalize_policy_value(value)
        if setting_name == 'check_user_name':
            return normalize_bool_setting_value(value)
        if setting_name in INTEGER_OPTIONS:
            return normalize_int_value(value)
        return value

    def query_value(self, setting_name, normalized_value):
        if setting_name == 'policy':
            return str(normalized_value).upper()
        if setting_name == 'check_user_name':
            return normalize_bool_setting_value(normalized_value)
        return normalized_value

    @staticmethod
    def format_set_query(variable_name, value, mode):
        return "SET %s %s = %s" % (mode.upper(), mysql_quote_identifier(variable_name, 'vars'), value)


def main():
    argument_spec = mysql_common_argument_spec()
    argument_spec.update(
        policy=dict(type='str', choices=['low', 'medium', 'strong']),
        length=dict(type='int'),
        mixed_case_count=dict(type='int'),
        number_count=dict(type='int'),
        special_char_count=dict(type='int'),
        check_user_name=dict(type='bool'),
        password_lifetime=dict(type='int', no_log=False),
        password_history=dict(type='int', no_log=False),
        reuse_interval=dict(type='int', no_log=False),
        mode=dict(type='str', choices=['global', 'persist'], default='global'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=[(
            'policy',
            'length',
            'mixed_case_count',
            'number_count',
            'special_char_count',
            'check_user_name',
            'password_lifetime',
            'password_history',
            'reuse_interval',
        )],
        supports_check_mode=True,
    )

    if mysql_driver is None:
        module.fail_json(msg=mysql_driver_fail_msg)

    try:
        cursor, _db_conn = mysql_connect(
            module,
            module.params['login_user'],
            module.params['login_password'],
            module.params['config_file'],
            module.params['client_cert'],
            module.params['client_key'],
            module.params['ca_cert'],
            'mysql',
            connect_timeout=module.params['connect_timeout'],
            check_hostname=module.params['check_hostname'],
        )
    except Exception as exc:
        module.fail_json(
            msg="unable to connect to database, check login_user and "
                "login_password are correct or %s has the credentials. "
                "Exception message: %s" % (module.params['config_file'], to_native(exc))
        )

    server_implementation = get_server_implementation(cursor)
    if server_implementation == 'mysql':
        server_version = get_server_version(cursor).split('-', 1)[0]
        if module.params['mode'] == 'persist' and LooseVersion(server_version) < LooseVersion('8.0'):
            module.fail_json(msg='mode=persist requires MySQL 8.0 or later.', changed=False)

    desired_settings = {
        'policy': module.params['policy'],
        'length': module.params['length'],
        'mixed_case_count': module.params['mixed_case_count'],
        'number_count': module.params['number_count'],
        'special_char_count': module.params['special_char_count'],
        'check_user_name': module.params['check_user_name'],
        'password_lifetime': module.params['password_lifetime'],
        'password_history': module.params['password_history'],
        'reuse_interval': module.params['reuse_interval'],
    }

    password_policy = MySQLPasswordPolicy(module, cursor, server_implementation)
    module.exit_json(
        **password_policy.configure(
            desired_settings,
            mode=module.params['mode'],
            check_mode=module.check_mode,
        )
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""
options:
  host:
    description:
    - IP Address or hostname of the ACI Multi Site Orchestrator host.
    - If the value is not specified in the task, the value of environment variable C(MSO_HOST) will be used instead.
    type: str
    aliases: [ hostname ]
  port:
    description:
    - Port number to be used for the REST connection.
    - The default value depends on parameter `use_ssl`.
    - If the value is not specified in the task, the value of environment variable C(MSO_PORT) will be used instead.
    type: int
  username:
    description:
    - The username to use for authentication.
    - If the value is not specified in the task, the value of environment variables C(MSO_USERNAME) or C(ANSIBLE_NET_USERNAME) will be used instead.
    type: str
  password:
    description:
    - The password to use for authentication.
    - If the value is not specified in the task, the value of environment variables C(MSO_PASSWORD) or C(ANSIBLE_NET_PASSWORD) will be used instead.
    type: str
  output_level:
    description:
    - Influence the output of this MSO module.
    - C(normal) means the standard output, incl. C(current) dict
    - C(info) adds informational output, incl. C(previous), C(proposed) and C(sent) dicts
    - C(debug) adds debugging output, incl. C(filter_string), C(method), C(response), C(status) and C(url) information
    - If the value is not specified in the task, the value of environment variable C(MSO_OUTPUT_LEVEL) will be used instead.
    type: str
    choices: [ debug, info, normal ]
    default: normal
  timeout:
    description:
    - The socket level timeout in seconds.
    - The default value is 30 seconds.
    - If the value is not specified in the task, the value of environment variable C(MSO_TIMEOUT) will be used instead.
    type: int
  use_proxy:
    description:
    - If C(false), it will not use a proxy, even if one is defined in an environment variable on the target hosts.
    - If the value is not specified in the task, the value of environment variable C(MSO_USE_PROXY) will be used instead.
    - The default is C(true).
    type: bool
  use_ssl:
    description:
    - If C(false), an HTTP connection will be used instead of the default HTTPS connection.
    - If the value is not specified in the task, the value of environment variable C(MSO_USE_SSL) will be used instead.
    - When using a HTTPAPI connection plugin the inventory variable C(ansible_httpapi_use_ssl) will be used if this attribute is not specified.
    - The default is C(false) when using a HTTPAPI connection plugin (mso or nd) and C(true) when using the legacy connection method (only for mso).
    type: bool
  validate_certs:
    description:
    - If C(false), SSL certificates will not be validated.
    - This should only set to C(false) when used on personally controlled sites using self-signed certificates.
    - If the value is not specified in the task, the value of environment variable C(MSO_VALIDATE_CERTS) will be used instead.
    - The default is C(true).
    type: bool
  login_domain:
    description:
    - The login domain name to use for authentication.
    - The default value is Local.
    - If the value is not specified in the task, the value of environment variable C(MSO_LOGIN_DOMAIN) will be used instead.
    - When using a HTTPAPI connection plugin the inventory variable C(ansible_httpapi_login_domain) will be used if this attribute is not specified.
    type: str
requirements:
- Multi Site Orchestrator v2.1 or newer
notes:
- This module was written to support Multi Site Orchestrator v2.1 or newer. Some or all functionality may not work on earlier versions.
"""



# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import errno
import os
import tempfile
import typing as t

if t.TYPE_CHECKING:  # pragma: no cover
    from ansible.module_utils.basic import AnsibleModule


def load_file(*, path: str | os.PathLike, module: AnsibleModule | None = None) -> bytes:
    """
    Load the file as a bytes string.
    """
    try:
        with open(path, "rb") as f:
            return f.read()
    except Exception as exc:
        if module is None:
            raise
        module.fail_json(f"Error while loading {path} - {exc}")


def load_file_if_exists(
    *,
    path: str | os.PathLike,
    module: AnsibleModule | None = None,
    ignore_errors: bool = False,
) -> bytes | None:
    """
    Load the file as a bytes string. If the file does not exist, ``None`` is returned.

    If ``ignore_errors`` is ``True``, will ignore errors. Otherwise, errors are
    raised as exceptions if ``module`` is not specified, and result in ``module.fail_json``
    being called when ``module`` is specified.
    """
    try:
        with open(path, "rb") as f:
            return f.read()
    except EnvironmentError as exc:
        if exc.errno == errno.ENOENT:
            return None
        if ignore_errors:
            return None
        if module is None:
            raise
        module.fail_json(f"Error while loading {path} - {exc}")
    except Exception as exc:
        if ignore_errors:
            return None
        if module is None:
            raise
        module.fail_json(f"Error while loading {path} - {exc}")


def write_file(
    *,
    module: AnsibleModule,
    content: bytes,
    default_mode: str | int | None = None,
    path: str | os.PathLike | None = None,
) -> None:
    """
    Writes content into destination file as securely as possible.
    Uses file arguments from module.
    """
    # Find out parameters for file
    try:
        file_args = module.load_file_common_arguments(module.params, path=path)
    except TypeError:
        # The path argument is only supported in Ansible 2.10+. Fall back to
        # pre-2.10 behavior of module_utils/crypto.py for older Ansible versions.
        file_args = module.load_file_common_arguments(module.params)
        if path is not None:
            file_args["path"] = path
    if file_args["mode"] is None:
        file_args["mode"] = default_mode
    # Create tempfile name
    tmp_fd, tmp_name = tempfile.mkstemp(prefix=b".ansible_tmp")
    try:
        os.close(tmp_fd)
    except Exception:
        pass
    module.add_cleanup_file(tmp_name)  # if we fail, let Ansible try to remove the file
    try:
        try:
            # Create tempfile
            file = os.open(tmp_name, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o600)
            os.write(file, content)
            os.close(file)
        except Exception as e:
            try:
                os.remove(tmp_name)
            except Exception:
                pass
            module.fail_json(msg=f"Error while writing result into temporary file: {e}")
        # Update destination to wanted permissions
        if os.path.exists(file_args["path"]):
            module.set_fs_attributes_if_different(file_args, False)
        # Move tempfile to final destination
        module.atomic_move(
            os.path.abspath(tmp_name), os.path.abspath(file_args["path"])
        )
        # Try to update permissions again
        if not module.check_file_absent_if_check_mode(file_args["path"]):
            module.set_fs_attributes_if_different(file_args, False)
    except Exception as e:
        try:
            os.remove(tmp_name)
        except Exception:
            pass
        module.fail_json(msg=f"Error while writing result: {e}")


__all__ = ("load_file", "load_file_if_exists", "write_file")
